/*
 CPLD Logic Analyser
 Version: 0.1
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 12/12/2013
 Last Modified: 12/12/2013
 
 This project uses an ATtiny84 with a Altera CPLD as a logic analyser and wrtiting the data to an external SRAM.
 At the moment, this version is for testing only.
 
 Plug in the USB and then run the laextract program to transfer the data to your computer.
  
 Remember to change your fuse bits so the ATtiny44A uses a 12MHz crystal
 avrdude -p ATtiny84 -c usbtiny -U lfuse:w:0xff:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m
 
 */

// ATtiny24/44/84 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//  XTAL   PB0  2|    |13  PA0 USB D+ 
//  XTAL   PB1  3|    |12  PA1 USB D-
//  Reset  PB3  4|    |11  PA2 
//  USB D+ PB2  5|    |10  PA3 CPLD Reset
//  SPI CS PA7  6|    |9   PA4 SPI SCK
//  SPI DI PA6  7|    |8   PA5 SPI DO
//               +----+
//

#define buttonPin PA2 // Input
#define cpldresetPin PA3 // Input/Output
#define usbminusPin PA1 // USB D-
#define spiCS PA7 // SPI Chip Select

#define F_CPU 12000000 // If this will be changed, modify it in usbconfig.h too

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include <string.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "setup.c"

boolean inusbTransfer = true;
uint16_t readCount = 0;
uint16_t endCount = 1000;
byte countDigits = 0;

// V-USB Transfer variables
#define USB_TRANSFER 1
static uchar replyBuf[16];
static uchar *nextDigit;
static uchar tempbuffer[16];
static uchar *nextDigittemp;
static uchar dataReceived = 0, dataLength = 0; // for USB_DATA_IN

// Convert a number to ASCII values and add it to the nextDigittemp variable
void numberToasciibuffer(uint32_t number) {
	uchar digit;
	do {
		countDigits++;
		digit = number % 10;
		number /= 10;
		if (digit == 0) {
			*--nextDigittemp = 0 + 48;
		}
		else {
			*--nextDigittemp = digit + 48;
		}
	} while (number != 0);
}

// SPI Transfer
uint8_t spi_transfer(uint8_t data) {
	USIDR = data;
	USISR = _BV(USIOIF); // clear flag

	while ( (USISR & _BV(USIOIF)) == 0 ) {
		USICR = (1<<USIWM0)|(1<<USICS1)|(1<<USICLK)|(1<<USITC);
	}
	return USIDR;
}

// Send the next digit from the address
int sendnextDigit(uint16_t sramAddress) {
	PORTA &= ~(1<<spiCS); // CS Low, active
	spi_transfer(0x03);
	spi_transfer(sramAddress >> 8);
	spi_transfer(sramAddress & 0xFF);
	int value = spi_transfer(0);
	PORTA |= (1<<spiCS); // CS High, in-active
	
	// Convert the number to ASCII and place it in a temporary buffer
	nextDigittemp = &tempbuffer[sizeof(tempbuffer)];
	countDigits = 0;
	numberToasciibuffer(value);
	
	// Transfer that temporary buffer to the reply buffer
	nextDigit = &replyBuf[0];
	memcpy(nextDigit, nextDigittemp, countDigits);
	nextDigit += countDigits;
	memcpy(nextDigit, "\r\n\0", 3);
	
	return 0;
}

// This gets called when custom control message is received
USB_PUBLIC uchar usbFunctionSetup(uchar data[8]) {
	usbRequest_t *rq = (void *)data; // cast data to correct type
	
	if (rq->bRequest == USB_TRANSFER) { // custom command is in the bRequest field
		nextDigit = &replyBuf[-1];
		if (readCount < endCount) {
			sendnextDigit(readCount);
			readCount = readCount + 1;
		}
		else {
			inusbTransfer = false;
		}
		
		usbMsgPtr = replyBuf;
		
		return sizeof(replyBuf);
	}
	return 0;
}

// This gets called when data is sent from PC to the device
USB_PUBLIC uchar usbFunctionWrite(uchar *data, uchar len) {
	uchar i;
	
	for(i = 0; dataReceived < dataLength && i < len; i++, dataReceived++)
		replyBuf[dataReceived] = data[i];
	
	return (dataReceived == dataLength); // 1 if we received it all, 0 if not
}

// Transfers the EEPROM data to USB.
void transferData (void) {
	
	// Reset variables
	inusbTransfer = true;
	readCount = 0;

	while (inusbTransfer == true) {
		usbPoll();
	}
}

int main (void) {
	setup();
	_delay_ms(500);
	
	// Keep CPLD in reset
	PORTA &= ~(1<<cpldresetPin);
	
	// Enable sequential mode by writing to the register
	PORTA &= ~(1<<spiCS); // CS Low, active
	spi_transfer(0x01);
	spi_transfer(0x40);
	PORTA |= (1<<spiCS); // CS High, in-active
	_delay_ms(10);
	
	// Begin a write to address 0 and then hand over to the CPLD to write the data
	PORTA &= ~(1<<spiCS); // CS Low, active
	spi_transfer(0x02);
	spi_transfer(0x00);
	spi_transfer(0x00);
	
	// CPLD reset high, let it write data
	PORTA |= (1<<cpldresetPin);
	
	_delay_ms(10);
	PORTA &= ~(1<<cpldresetPin);
	PORTA |= (1<<spiCS); // CS High, in-active
	
	
	
	// Test used for writing directly
	/*PORTA &= ~(1<<cpldresetPin);
	PORTA &= ~(1<<spiCS); // CS Low, active
	spi_transfer(0x01);
	spi_transfer(0x40);
	PORTA |= (1<<spiCS); // CS High, in-active
	_delay_ms(10);
	
	PORTA &= ~(1<<spiCS); // CS Low, active
	spi_transfer(0x02);
	spi_transfer(0x00);
	spi_transfer(0x00);
	spi_transfer(0x01);
	spi_transfer(0x02);
	spi_transfer(0x03);
	spi_transfer(0x04);
	spi_transfer(0x05);
	spi_transfer(0x06);
	spi_transfer(0x07);
	spi_transfer(0x08);
	spi_transfer(0x09);
	spi_transfer(0x010);
	spi_transfer(0x011);
	spi_transfer(0x012);
	spi_transfer(0x013);
	spi_transfer(0x014);
	spi_transfer(0x015);
	PORTA |= (1<<spiCS); // CS High, in-active
	_delay_ms(10);
	*/
	
	
	startUSB();
	transferData();
	
	return 0;
}
